%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM. Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%%  Copyright IBM Corporation 2011 
%% U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

% ================================================================
% --- Executes on Import button press
% ================================================================
function dm_publisher_win(model, serverURI, projArea, dmUser, dmPw, scriptFolder, doDelete)

	password = dmPw;
	user = dmUser;
	path2mdl = model;
	DMProjectArea = projArea;
	dmuri = serverURI;

	% Prepare variables:
	javalib = strcat(scriptFolder, '/domains/simulink/bin');
	fprintf('javalib=%s\n', javalib);
	files = dir(javalib);
	nfiles = size(files);
	pubDir = strcat(tempdir, 'SimulinkImporterData');

	% ================================================================
	% PRINT INPUT PARAMETERS
	% ================================================================
	fprintf('\n===========================\n');
	fprintf('Input Parameters:\n');
	fprintf('===========================\n');
	fprintf('Import directory=%s\n', pubDir);
	fprintf('Simulink model file=%s\n', path2mdl);
	fprintf('RDM server address=%s\n', dmuri);
	fprintf('RDM project area=%s\n', DMProjectArea);
	fprintf('===========================\n\n');

	% ================================================================
	% EXECUTE IMPORT
	% ================================================================
	[s,mess,messid] = mkdir(pubDir);

	sys = load_system(path2mdl);
	if isobject(sys) == 0
		
		% add libraries to java path
		warning('off', 'MATLAB:javaclasspath:jarAlreadySpecified');
		for x = 1:nfiles
			filename = files(x).name;
			if strcmp(filename,'.') == 0 && strcmp(filename,'..') == 0
				javaaddpath(strcat(javalib,'/', filename));
			end
		end
		warning('on', 'MATLAB:javaclasspath:jarAlreadySpecified');
		
		tempSysName = get_param(sys,'name');
		sysName = tempSysName;
		pubDir = strcat(pubDir, '/',sysName);
		[s,mess,messid] = mkdir(pubDir);

		% Prepare java objects:
		publisher = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.Publisher', pubDir, dmuri, DMProjectArea, user, password);
		if isjava(publisher) == 0 
			fprintf('ERROR: java object (Publisher) allocation failed\n');
		end
		
		helper = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.MscriptHelper');
		if isjava(helper) == 0 
			fprintf('ERROR: java object (MscriptHelper) allocation failed\n');
		end
		
		publisher.setModelFile(path2mdl);
		publisher.createModel(sysName);
		
		getSystemData(sys, publisher, '', helper, pubDir);
		
		% Serialize and publish to RDM
		publisher.Serialize();
		publisher.publish();
	   
		close_system(path2mdl, 0);
		
		fprintf('Done\n');
		
	end
end

% ================================================================
% getSystemData
% ================================================================
function getSystemData(sys, publisher, parentSysPath, helper, publishDir)
    
    internalSysName = get_param(sys,'name');
    sysName = regexprep(internalSysName, '\n', ' ');
    sysDescript = get_param(sys,'Description');
    
    sysUriName = char(helper.legalizeMdlElemName(sysName));
    
    subSystem = 0;

     if ~isempty(parentSysPath)
         publisher.addSystem(sysName, sysUriName, sysDescript, 0);
         subSystem = 1;
     else
         publisher.addSystem(sysName, sysUriName, sysDescript, 1);
     end
    
    legalParentSysPath = regexprep(parentSysPath, '\n', ' ');
    
    sysPath = sysName;
    if isempty(legalParentSysPath) == 0
        sysPath = strcat(legalParentSysPath,'/',sysName);
    end
        
	% save the model file in the publishDir
		ext = 'mdl';
	try
		ext = get_param(0, 'ModelFileFormat');
	catch
	end
	filename = strcat(internalSysName,'.',ext);
	fullFileName = strcat(publishDir,'/',filename);
	try
		save_system(sys,fullFileName);
	catch
	end
	
    % Print diagram:
    % ======================
    isStateFlow = '';
    try
        isStateFlow = get_param(sysPath, 'MaskType');
    catch
        fprintf('No MaskType found for %s\n', sysPath);
    end
    
    if strcmp(isStateFlow, 'Stateflow') == 1
        printChartDiagram(sys, sysPath, publishDir, helper);
    else
        printSystemDiagram(sys, publishDir, helper);
    end
    
    % Get system lines:
    % ======================
    SysLines = get_param (sys,'Lines');
    numOfLines = size(SysLines);
    
    for z = 1:numOfLines
        lineName = SysLines(z).Name;
        srcBlock = get_param(SysLines(z).SrcBlock, 'Name');
        legalSrcBlock = char(helper.legalizeMdlElemName(srcBlock));
        destBlock = get_param(SysLines(z).DstBlock, 'Name');
        legalDestBlock = char(helper.legalizeMdlElemName(destBlock));
        
        publisher.addSignal(lineName, legalSrcBlock, legalDestBlock);
    end
    
    % Get system blocks:
    % ======================
    blocks = get_param (sys,'Blocks');
    numOfBlocks = size(blocks);

    fprintf('*** blocks Iteration ***\n');
    
    for x = 1:numOfBlocks
        
        tempBlockName = blocks{x};
        internalBlockName = regexprep(tempBlockName, '/', '//');
        legalBlockName = regexprep(internalBlockName, '\n', ' ');
        tempBlockPath = strcat(sysPath,'/',legalBlockName);
        internalBlockPath = horzcat(sysPath, '/',legalBlockName);
        blockPath = regexprep(tempBlockPath, '\n', ' ');
        
        if subSystem == 1
            legalSysPath = char(helper.legalizeMdlElemName(sysPath));
            legalBlockName2 = char(helper.legalizeMdlElemName(legalBlockName));
            uriName = strcat(legalSysPath, '_subsys_', legalBlockName2);
        else
            uriName = char(helper.legalizeMdlElemName(blockPath));
        end
        
        blockPosition = get_param (internalBlockPath,'Position');
        blockType = get_param (internalBlockPath,'BlockType');
        blockDesc = get_param (internalBlockPath,'Description');
        
        dblArray = javaArray ('java.lang.Double', 4);

        dblArray(1) = java.lang.Double(blockPosition(1));
        dblArray(2) = java.lang.Double(blockPosition(2));
        dblArray(3) = java.lang.Double(blockPosition(3));
        dblArray(4) = java.lang.Double(blockPosition(4));
        
        publisher.addBlock(sysUriName, legalBlockName, uriName, blockType, blockDesc, dblArray);
        
        ObjParamsStruct = get_param(internalBlockPath, 'DialogParameters');
        numOfParams = 0;
        try
            ObjParams = struct2cell(ObjParamsStruct);
            InternalNames = fieldnames(ObjParamsStruct);
            numOfParams = size(ObjParams);
        catch
            fprintf('ERROR: struct2cell() failure\n');
        end
         
        getProps = 1;
        if strcmp(blockType, 'Scope') == 1
            getProps = 0;
        elseif numOfParams < 1
            getProps = 0;
        end
        
        % Get block parameters:
        % ======================
        if getProps > 0
            for y = 1:numOfParams                
                paramName = ObjParams{y}.Prompt;                
                paramVal = get_param(internalBlockPath, InternalNames{y});
                if ischar(paramName) && ischar(paramVal)
                    publisher.addProperty(paramName, paramVal);
                end
            end
        end
        
        % get sub system data:
        % ======================
        if strcmp(blockType, 'SubSystem') == 1
            subsys = find_system(sys, 'name', internalBlockName);
            if isobject(subsys) == 0
                openFunc = get_param(subsys, 'OpenFcn');
                if isempty(openFunc)
                    getSystemData(subsys, publisher, sysPath, helper, publishDir);
                end
            end
        end        
    end
end
    
function printSystemDiagram(sys, publishDir, helper)

    orient(sys,'landscape');

    internalSysName = get_param(sys,'name');
    tempSysUriName = strrep(internalSysName, '/', '_');
    sysUriName = char(helper.legalizeMdlElemName(tempSysUriName));
    set(sys, 'PaperOrientation' , 'portrait');
    diagPicDir = strcat(publishDir,'/', sysUriName ,'.bmp');
    saveas(sys, diagPicDir);
end
     
function printChartDiagram(sys, sysPath, publishDir, helper)

    orient(sys,'landscape');

    internalSysName = get_param(sys,'name');
    tempSysUriName = strrep(internalSysName, '/', '_');
    sysUriName = char(helper.legalizeMdlElemName(tempSysUriName));
    set(sys, 'PaperOrientation' , 'portrait');
    diagPicDir = strcat(publishDir,'/', sysUriName ,'.bmp');
    sfprint(sysPath,'bitmap',diagPicDir);
end