function saveSimulinkModelToDM(bMoveToDM)
	try
		evalin('base', 'createDMConnection');
		dmConn = evalin('base', 'dmConnection');
	catch
		return;
	end	
	
	if isjava(dmConn) == 0
		return;
	end	
	
	%check if we need to save and if model is locked on server before save
	if bMoveToDM == 0
		dirtyFlag = get_param(gcs,'dirty');
		if strcmp(dirtyFlag, 'off') == 1
			return;
		end	
		simMdlName = get_param(gcs, 'name');
		if dmConn.canSaveSimulinkModelOnDM(simMdlName) == 0
			return;
		end	
		
		%clear all callbacks before we save on DM
		set_param(gcs,'PostLoadFcn','');
		set_param(gcs,'PreSaveFcn','');
		set_param(gcs,'PostSaveFcn','');
		set_param(gcs,'CloseFcn','');
	end
	
	moveToAction = '';
	if dmConn.isConnectionAvailable() == 0 || bMoveToDM == 1
		moveToAction = char(dmConn.openMoveToDMDialog());
		if isempty(moveToAction) == 1
			return;
		end		
	end
	
	path2mdl = which(gcs);

	pubDir = strcat(tempdir, 'SimulinkImporterData');
	[s,mess,messid] = mkdir(pubDir);

	sys = load_system(path2mdl);
	if isobject(sys) == 0
		tempSysName = get_param(sys,'name');
		
		if bMoveToDM == 1
			msg = sprintf('Moving %s to DM...',tempSysName); 
		else
			msg = sprintf('Saving %s to DM...',tempSysName); 
		end	
		disp(msg);
		
		sysName = tempSysName;
		pubDir = strcat(pubDir, '/',sysName);
		[s,mess,messid] = mkdir(pubDir);

		% Prepare java objects:
		publisher = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.Publisher', pubDir);
		if isjava(publisher) == 0 
			fprintf('ERROR: java object (Publisher) allocation failed\n');
			return;
		end
    
		helper = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.MscriptHelper');
		if isjava(helper) == 0 
			fprintf('ERROR: java object (MscriptHelper) allocation failed\n');
			return;
		end
    
		publisher.setModelFile(path2mdl);
		publisher.createModel(sysName);
    
		getSystemData(sys, publisher, '', helper, pubDir);
    
		% Serialize and publish to DM
		publisher.Serialize();
		bSuccess = publisher.moveToDM();
		if bSuccess == 1
			set_param(gcs,'PostSaveFcn','postOpenFromDMActions()');
			save_system(gcs);
			
			if bMoveToDM == 1	
				disp('Move to done.');
			else	
				disp('Saving done.');
			end
			
			%last, lock the model since save/moveto release the locking
			if bMoveToDM == 0 || strcmp(moveToAction, 'MoveToAndLock') == 1
				dmConn.lockSimulinkModel(sysName);
			end	
		else
			if bMoveToDM == 1
				disp('Failed to move model to DM');
			else
				disp('Failed to save model to DM');
			end	
		end		
	end
end

function importerDir = getImporterFolder()
    myPath = mfilename('fullpath');
    slashedPath = regexprep(myPath, '\\', '/');
    importerDir = regexprep(slashedPath, 'mscripts/saveSimulinkModelToDM', '');
end

function getSystemData(sys, publisher, parentSysPath, helper, publishDir)
    
    internalSysName = get_param(sys,'name');
    sysName = regexprep(internalSysName, '\n', ' ');
    sysDescript = get_param(sys,'Description');
    
    sysUriName = char(helper.legalizeMdlElemName(sysName));
    
    subSystem = 0;

     if ~isempty(parentSysPath)
         publisher.addSystem(sysName, sysUriName, sysDescript, 0);
         subSystem = 1;
     else
         publisher.addSystem(sysName, sysUriName, sysDescript, 1);
     end
    
    legalParentSysPath = regexprep(parentSysPath, '\n', ' ');
    
    sysPath = sysName;
    if isempty(legalParentSysPath) == 0
        sysPath = strcat(legalParentSysPath,'/',sysName);
    end
        
	% save the model file in the publishDir
	ext = 'mdl';
	try
		ext = get_param(0, 'ModelFileFormat');
	catch
	end
	filename = strcat(internalSysName,'.',ext);
	fullFileName = strcat(publishDir,'/',filename);
	try
		save_system(sys,fullFileName);
	catch
	end
	
    % Print diagram:
    % ======================
    isStateFlow = '';
    try
        isStateFlow = get_param(sysPath, 'MaskType');
    catch
  %      fprintf('No MaskType found for %s\n', sysPath);
    end
    
    if strcmp(isStateFlow, 'Stateflow') == 1
        printChartDiagram(sys, sysPath, publishDir, helper);
    else
        printSystemDiagram(sys, publishDir, helper);
    end
    
    % Get system lines:
    % ======================
    SysLines = get_param (sys,'Lines');
    numOfLines = size(SysLines);
    
    for z = 1:numOfLines
        lineName = SysLines(z).Name;
        srcBlock = get_param(SysLines(z).SrcBlock, 'Name');
        legalSrcBlock = char(helper.legalizeMdlElemName(srcBlock));
        destBlock = get_param(SysLines(z).DstBlock, 'Name');
        legalDestBlock = char(helper.legalizeMdlElemName(destBlock));
        
        publisher.addSignal(lineName, legalSrcBlock, legalDestBlock);
    end
    
    % Get system blocks:
    % ======================
    blocks = get_param (sys,'Blocks');
    numOfBlocks = size(blocks);

    for x = 1:numOfBlocks
        
        tempBlockName = blocks{x};
        internalBlockName = regexprep(tempBlockName, '/', '//');
        legalBlockName = regexprep(internalBlockName, '\n', ' ');
        tempBlockPath = strcat(sysPath,'/',legalBlockName);
        internalBlockPath = horzcat(sysPath, '/',legalBlockName);
        blockPath = regexprep(tempBlockPath, '\n', ' ');
        
        if subSystem == 1
            legalSysPath = char(helper.legalizeMdlElemName(sysPath));
            legalBlockName2 = char(helper.legalizeMdlElemName(legalBlockName));
            uriName = strcat(legalSysPath, '_subsys_', legalBlockName2);
        else
            uriName = char(helper.legalizeMdlElemName(blockPath));
        end
        
        blockPosition = get_param (internalBlockPath,'Position');
        blockType = get_param (internalBlockPath,'BlockType');
        blockDesc = get_param (internalBlockPath,'Description');
        
        dblArray = javaArray ('java.lang.Double', 4);

        dblArray(1) = java.lang.Double(blockPosition(1));
        dblArray(2) = java.lang.Double(blockPosition(2));
        dblArray(3) = java.lang.Double(blockPosition(3));
        dblArray(4) = java.lang.Double(blockPosition(4));
        
        publisher.addBlock(sysUriName, legalBlockName, uriName, blockType, blockDesc, dblArray);
        
        ObjParamsStruct = get_param(internalBlockPath, 'DialogParameters');
        numOfParams = 0;
        try
            ObjParams = struct2cell(ObjParamsStruct);
            InternalNames = fieldnames(ObjParamsStruct);
            numOfParams = size(ObjParams);
        catch
            fprintf('ERROR: struct2cell() failure\n');
        end
         
        getProps = 1;
        if strcmp(blockType, 'Scope') == 1
            getProps = 0;
        elseif numOfParams < 1
            getProps = 0;
        end
        
        % Get block parameters:
        % ======================
        if getProps > 0
            for y = 1:numOfParams                
                paramName = ObjParams{y}.Prompt;                
                paramVal = get_param(internalBlockPath, InternalNames{y});
                if ischar(paramName) && ischar(paramVal)
                    publisher.addProperty(paramName, paramVal);
                end
            end
        end
        
        % get sub system data:
        % ======================
        if strcmp(blockType, 'SubSystem') == 1
            subsys = find_system(sys, 'name', internalBlockName);
            if isobject(subsys) == 0
                openFunc = get_param(subsys, 'OpenFcn');
                if isempty(openFunc)
                    getSystemData(subsys, publisher, sysPath, helper, publishDir);
                end
            end
        end        
    end
end
    
function printSystemDiagram(sys, publishDir, helper)
    orient(sys,'landscape');

    internalSysName = get_param(sys,'name');
    tempSysUriName = strrep(internalSysName, '/', '_');
    sysUriName = char(helper.legalizeMdlElemName(tempSysUriName));
%	oldOrientation = get(sys,'PaperOrientation');
 %   set(sys, 'PaperOrientation' , 'portrait');
    diagPicDir = strcat(publishDir,'/', sysUriName ,'.bmp');
    saveas(sys, diagPicDir);
%	set(sys, 'PaperOrientation', oldOrientation);
%	save_system(sys);
end
     
function printChartDiagram(sys, sysPath, publishDir, helper)
    orient(sys,'landscape');

    internalSysName = get_param(sys,'name');
    tempSysUriName = strrep(internalSysName, '/', '_');
    sysUriName = char(helper.legalizeMdlElemName(tempSysUriName));
%	oldOrientation = get(sys,'PaperOrientation');
 %   set(sys, 'PaperOrientation' , 'portrait');
    diagPicDir = strcat(publishDir,'/', sysUriName ,'.bmp');
    sfprint(sysPath,'bitmap',diagPicDir);
%	set(sys, 'PaperOrientation', oldOrientation);
%	save_system(sys);
end