%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM. Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%%  Copyright IBM Corporation 2011 
%% U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

function sl_customization(cm)

  %% Register custom menu function.
  cm.addCustomMenuFcn('Simulink:ToolsMenu', @getMyMenuItems);
end

%% Define the custom menu function.
function schemaFcns = getMyMenuItems(callbackInfo) 
  schemaFcns = {@menuRDM,...
                }; 
end

%% Define the schema function for group menu item.
function schema = menuRDM(callbackInfo)
  schema = sl_container_schema;
  schema.label = 'Rhapsody Design Manager';
  schema.childrenFcns = {@menuPublish, @menuNavigateToServer, 'separator', @menuMoveToDM, @menuLockModel, @menuUnlockModel, @menuDisconnectModel};
end

function schema = menuPublish(callbackInfo)
  schema = sl_action_schema;
  schema.label = 'Publish model...';
  schema.userdata = 'publish model to RDM server';	
  schema.callback = @publishToServer;
end 

function publishToServer(callbackInfo)
    
    loadJavaObjects();

    launcher = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.DMLauncher');
     if isjava(launcher) == 0 
        fprintf('ERROR: java object (launcher) allocation failed\n');
     else
        path = which (bdroot);
        importerDir = getImporterFolder();
        launcher.launchImporter(importerDir, path);
     end
end

function schema = menuNavigateToServer(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Navigate to Design Manager...';
	schema.userdata = 'navigate to server';	
	schema.callback = @navigateToServer;
end 

function navigateToServer(callbackInfo)
    
    loadJavaObjects();

    launcher = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.DMLauncher');
     if isjava(launcher) == 0 
        fprintf('ERROR: java object (launcher) allocation failed\n');
     else
        path = which (bdroot);
        importerDir = getImporterFolder();
        
        sys = load_system(path);
        name = get_param(sys, 'name');

        launcher.launchDM(importerDir, name);
     end
end

function schema = menuMoveToDM(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Move to Design Manager...';
	schema.userdata = 'Move to Design Manager...';	
	schema.callback = @moveToDM;
end

function moveToDM(callbackInfo)
	saveSimulinkModelToDM(1);
end

function schema = menuSaveOnDM(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Save to server';
	schema.userdata = 'save to server';	
	schema.callback = @saveToDM;
end

function saveToDM(callbackInfo)
	saveSimulinkModelToDM(0);
end

function schema = menuLockModel(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Lock Model';
	schema.userdata = 'Lock Model';	
	schema.callback = @lockModelInDM;
end

function lockModelInDM(callbackInfo)
	lockSimulinkModelInDM();
end

function schema = menuUnlockModel(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Unlock Model';
	schema.userdata = 'Unlock Model';	
	schema.callback = @unlockModelInDM;
end

function unlockModelInDM(callbackInfo)
	unlockSimulinkModelInDM();
end

function schema = menuDisconnectModel(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Disconnect Model...';
	schema.userdata = 'Disconnect Model...';	
	schema.callback = @disconnectSimulinkModelFromDM;
end
	


function importerDir = getImporterFolder()
    myPath = mfilename('fullpath');
    slashedPath = regexprep(myPath, '\\', '/');
    importerDir = regexprep(slashedPath, 'mscripts/sl_customization', '');
end

